/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* Packout Tumbler Handle
* Benjamen Johnson <workshop.electronsmith.com>
* 20220506
* openSCAD Version: 2021.01 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
version="V0_2";

//Define the roundness of curves
$fa = 2;
$fs = 0.1;
$dl = 0.02;

/*[Calibration]*/
/******************************************************************************
*******************************************************************************/
// Pick which object to make
object = "Tumbler"; //["Tumbler","Calibration Ring"]

// Adjust height of handle up ore down on the tumbler
ring_height_adj = 8;

/*[Tumbler Parameters]*/
/******************************************************************************
*******************************************************************************/
// Diameter at the top of the tumbler
top_dia = 88; //was 87.5;

// Diameter at the transition
ridge_dia = 86; //was 85.5;

// Distance between the transition and the top of the tumbler
top_to_ridge_height = 75;

// Figure out the slope of the top section of the tumbler
run_rise = (top_dia-ridge_dia)/top_to_ridge_height/2;

/*[Tumbler Ring Parameters]*/
/******************************************************************************
*******************************************************************************/
// How thick are the walls of the holder
wall_thickness = 3;

// Position of the Milwaukee Window with the handle to the right
window_pos = "Facing"; //["Facing","Away"]

// Height of the bottom edge of the handle holder
bottom_height = 5;

// Height of the top edge of the handle holder
top_height = 60;

b_h = bottom_height+ring_height_adj;
t_h = top_height+ring_height_adj;

/*[Hidden]*/
/******************************************************************************
* Handle Parameters
*******************************************************************************/
handle_dia1 = 20;
handle_dia2 = 40;
handle_offset = -15;
width_multiplier = 1.25 ;

handle_width = handle_dia1*width_multiplier;

handle_thickness = handle_dia1/2+handle_dia2/2+handle_offset;

handle_length = 70;   

bend_radius = 12;

segment_height = 1;
segment_angle = 2;   


ring_height = 50;
ring_width = 10;

/*******************************************************************************
Create the tumbler handle
*******************************************************************************/
if(object == "Tumbler")
rotate([180,0,0])    
union(){
    // show the top section of the tumbler
    translate([0,0,-b_h])
    %cylinder(h=top_to_ridge_height, d2=top_dia, d1=ridge_dia);

    difference(){
        union(){
            // Make the ring minus the chamfer
            translate([0,0,wall_thickness])
            cylinder(h=(t_h-b_h-wall_thickness),d1 = cup_dia(b_h+wall_thickness)+2*wall_thickness,d2=cup_dia(t_h)+2*wall_thickness);
            
            // Chamfer bottom of the ring 
            cylinder(h=wall_thickness,d2=cup_dia(b_h)+2*wall_thickness,d1=cup_dia(b_h));
            
            // Make long part of handle
            translate([0,87,(t_h-b_h)-handle_length-bend_radius-segment_height-handle_thickness]) //-42
            linear_extrude(height = handle_length,convexity = 10)
                shape();
            
            // Make outside handle bend
            translate([0,63,26])
            rotate([0,0,-90])
            bend(90,bend_radius);

            // Make inside handle bend
            translate([0,61,26])
            rotate([0,0,90])
            bend(90,bend_radius);

            // Make top of handle
            translate([0,62,50]) //50
            rotate([90,0,0])
            linear_extrude(height = 25,convexity = 10)
                shape();
        }//union
        
        // hollow out handle ring
        translate([0,0,-$dl/2 ])
        cylinder(h=(t_h-b_h)+$dl,d1 = cup_dia(b_h),d2=cup_dia(t_h));   
        
        // make window
        if(window_pos == "Facing")
            translate([40,0,-b_h+(b_h+t_h)/2])
            cube([30,80,35],center=true);
        else
            translate([-40,0,-b_h+(b_h+t_h)/2])
            cube([30,80,35],center=true);
    }//difference
} // end union
else
    rotate([180,0,0])
    union(){
        // show the top section of the tumbler
    translate([0,0,-(ring_height_adj+ring_height-ring_width)])
    %cylinder(h=top_to_ridge_height, d2=top_dia, d1=ridge_dia);

    difference(){
        upper_inner_dia = cup_dia(ring_height_adj+ring_height);
        lower_inner_dia = cup_dia(ring_height_adj+ring_height-ring_width);
        upper_outer_dia = upper_inner_dia +2*wall_thickness;
        lower_outer_dia = lower_inner_dia +2*wall_thickness;
        
        cylinder(h=ring_width,d1 = lower_outer_dia,d2=upper_outer_dia);
        
        translate([0,0,-$dl/2 ])
        cylinder(h=(ring_width)+$dl,d1 =lower_inner_dia,d2=upper_inner_dia);
        }//difference
    }
/*******************************************************************************
* Create the handle shape
*******************************************************************************/
module shape(){
    
    scale([width_multiplier,1,1])
    intersection(){
        circle(d=handle_dia1);
        
        translate([0,handle_offset,0])
        circle(d=handle_dia2);
    }//intersection
}//module

/*******************************************************************************
* Create a bent shape
*******************************************************************************/
module bend(angle,radius){
   
    for(i=[0:segment_angle:angle])
        rotate([0,i,0])
        translate([-2*radius,0,0])
        linear_extrude(height = segment_height,convexity = 10)
        rotate([0,0,90])
            shape();
} //module

/*******************************************************************************
* Return the diameter of the cup at a certain height
*******************************************************************************/
function cup_dia(h) = 2*run_rise*h+ridge_dia;